"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourcePointer = exports.PointerType = void 0;
const resource_type_1 = require("./resource-type");
var PointerType;
(function (PointerType) {
    PointerType["EPHEMERAL"] = "EPHEMERAL";
    PointerType["PERMANENT"] = "PERMANENT";
    PointerType["ASSIGNABLE"] = "ASSIGNABLE";
})(PointerType = exports.PointerType || (exports.PointerType = {}));
const ALPHANUMERIC = /^[a-zA-Z0-9._\~\-]+$/;
class ResourcePointer {
    constructor(datasetId = "", resourceType = null, pointerType = null, id = "") {
        this.datasetId = datasetId;
        this.resourceType = resourceType;
        this.pointerType = pointerType;
        this.id = id;
    }
    get resourceShowId() {
        return (ResourcePointer.toResourceTypeCode(this.resourceType) +
            ResourcePointer.toPointerTypeCode(this.pointerType) +
            this.id);
    }
    static assignablePtr(datasetId, resourceType, id) {
        return new ResourcePointer(datasetId, resourceType, PointerType.EPHEMERAL, id);
    }
    static permanentPtr(datasetId, resourceType, id) {
        return new ResourcePointer(datasetId, resourceType, PointerType.PERMANENT, id);
    }
    static equals(one, two) {
        if (!one || !two)
            return !one && !two;
        return one.equals(two);
    }
    static listEquals(one, two) {
        if (!one || !two)
            return !one && !two;
        if (one.length !== two.length)
            return false;
        for (let i = 0; i < one.length; i++)
            if (!ResourcePointer.equals(one[i], two[i]))
                return false;
        return true;
    }
    static fromStringArr(arr, defaultDatasetId) {
        return arr.map((s) => ResourcePointer.fromString(s, defaultDatasetId));
    }
    static fromString(s, defaultDatasetId) {
        if (!s)
            return null;
        s = s.trim();
        if (s.startsWith("#"))
            s = s.substring(1);
        const parts = s.split("/");
        if (parts.length < 1 || parts.length > 2)
            return null;
        const datasetIdPart = parts.length === 1 ? defaultDatasetId : parts[0];
        const resourceIdPart = parts[parts.length - 1];
        if (!datasetIdPart ||
            !resourceIdPart ||
            datasetIdPart.length < 3 ||
            resourceIdPart.length < 3) {
            return null;
        }
        if (!ALPHANUMERIC.test(datasetIdPart) ||
            !ALPHANUMERIC.test(resourceIdPart)) {
            return null;
        }
        const rType = ResourcePointer.toResourceType(resourceIdPart.charAt(0));
        if (!rType)
            return null;
        const pType = ResourcePointer.toPointerType(resourceIdPart.charAt(1));
        if (!pType)
            return null;
        return new ResourcePointer(datasetIdPart, rType, pType, resourceIdPart.substr(2));
    }
    static isValid(ptr) {
        return !!(ptr &&
            ptr.datasetId &&
            ptr.pointerType &&
            ptr.resourceType &&
            ptr.id);
    }
    static toResourceType(char) {
        switch (char) {
            case "T":
                return resource_type_1.ResourceType.TERM;
            case "P":
                return resource_type_1.ResourceType.PROPOSITION;
            case "A":
                return resource_type_1.ResourceType.ARGUMENT;
            case "B":
                return resource_type_1.ResourceType.BELIEFSET;
            case "R":
                return resource_type_1.ResourceType.ARTICLE;
            case "J":
                return resource_type_1.ResourceType.PROJECT;
            case "M":
                return resource_type_1.ResourceType.MACHINE;
        }
        return null;
    }
    static toPointerType(char) {
        switch (char) {
            case ".":
                return PointerType.PERMANENT;
            case "_":
                return PointerType.ASSIGNABLE;
            case "~":
                return PointerType.EPHEMERAL;
        }
        return null;
    }
    static toPointerTypeCode(type) {
        switch (type) {
            case PointerType.PERMANENT:
                return ".";
            case PointerType.ASSIGNABLE:
                return "_";
            case PointerType.EPHEMERAL:
                return "~";
        }
        return null;
    }
    static toResourceTypeCode(type) {
        switch (type) {
            case resource_type_1.ResourceType.TERM:
                return "T";
            case resource_type_1.ResourceType.PROPOSITION:
                return "P";
            case resource_type_1.ResourceType.ARGUMENT:
                return "A";
            case resource_type_1.ResourceType.BELIEFSET:
                return "B";
            case resource_type_1.ResourceType.ARTICLE:
                return "R";
            case resource_type_1.ResourceType.PROJECT:
                return "J";
            case resource_type_1.ResourceType.MACHINE:
                return "M";
        }
        return null;
    }
    toString(defaultDatasetId) {
        if (!ResourcePointer.isValid(this))
            return "";
        if (defaultDatasetId === this.datasetId)
            return this.resourceShowId;
        return this.datasetId + "/" + this.resourceShowId;
    }
    toDisplayString(defaultDatasetId) {
        if (!ResourcePointer.isValid(this))
            return "";
        const strippedResourceShowId = this.resourceShowId.replace(/^(\w~)[^~]+~(.+)$/, "$1$2");
        if (defaultDatasetId === this.datasetId)
            return strippedResourceShowId;
        return this.datasetId + "/" + strippedResourceShowId;
    }
    equals(that) {
        if (!that) {
            return false;
        }
        return (this.datasetId === that.datasetId &&
            this.resourceType === that.resourceType &&
            this.pointerType === that.pointerType &&
            this.id === that.id);
    }
    isPermanentPtr() {
        return this.pointerType === PointerType.PERMANENT;
    }
    isAssignablePtr() {
        return this.pointerType === PointerType.ASSIGNABLE;
    }
    isEphemeralPtr() {
        return this.pointerType === PointerType.EPHEMERAL;
    }
}
exports.ResourcePointer = ResourcePointer;
